#!/bin/sh -- # -*- perl -*- -w
eval 'exec perl -S $0 ${1+"$@"}'
    if 0;

#
# Copyright 1997, Swiss Federal Institute of Technology, Zuerich (ETHZ)
#
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of ETHZ not be used in advertising or
# publicity pertaining to distribution of the software without specific,
# written prior permission.  ETHZ makes no representations about the
# suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
#
# ETHZ DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL ETHZ
# BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
# OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#
#-----------------------------------------------------------------------------
# Project:               Crossbow (Originated from Da CaPo++)
#-----------------------------------------------------------------------------
#
# File:			$Source: /home/arl/arlsw/mwa/Admin/cvsroot/home/mwa/src/bmp/v2/error/Config2h.pl,v $
# Author(s):		Marcel Waldvogel
# Affiliation:		ETH Zuerich, TIK
# Version:		$Revision: 1.4 $
# Creation Date:	2.7.97
# Last Changed:		$Date: 1999/01/12 18:04:31 $ by $Author: mwa $
#-----------------------------------------------------------------------------
# 
# * Concept and Idea:
#	Gives the possibility for an abstract error definition file
#
# * Performed Functions:
#	Converts Error.config to ErrorConfig.h.
#
# * Usage:
#	Config2h.pl [-v] [-c <configfile> ...] [<infile> <outfile> ...]
#		-v turns on verbose mode
#		-c reads in a config file (any number of times)
#		<infile> <outfile> pairs: Convert infile to outfile
#
#-----------------------------------------------------------------------------
#
# * Solved Errors (Function, Type of Error, Solution, Author, Date):
#-----------------------------------------------------------------------------
#
# * Remarks:
#
#=============================================================================

# Initialize arguments to be read from config file
$MacroPrefix = "";
$ErrorClassPrefix = "";
$ErrorSeverityEnumPrefix = "";
$ErrorSeverityMacroPrefix = "";
@ErrorLevels=();
$DefaultLoggingLevel="";
$SetAllName="";
@Subsystems=();
%LogPriorityMapping=();
$MinLogCompatibility="";
$MaxLogCompatibility="";


if ($ARGV[0] eq "-v") {
    $verbose=1;
    shift();
} else {
    $verbose=0;
}

while ($ARGV[0] eq "-c") {
    shift();
    require shift();
}

# Now we need pairs of file names
die "Input/output filenames not pairwise" if ((@ARGV & 1) != 0 && @ARGV > 0);

while (@ARGV > 0) {
    $infile = shift();
    open(INFILE, "<$infile") || die "Cannot open $infile for reading";
    $outfile = shift();
    open(OUTFILE, ">$outfile") || die "Cannot open $outfile for writing";
    $linenum=1;
    print OUTFILE "#line $linenum \"$infile\"\n";
    while (<INFILE>) {
	chop();
	$linenum++;
	if (/^#GENERATE/) {
	    print OUTFILE
		"#line 1 \"$infile/$linenum/$_\"\n",
		"/* START $_ */\n";
	}

	if (/^#GENERATE ER_SEV_DEFINES$/) {
	    my $i=0;
	    my $symbol;
	    for $symbol (@ErrorLevels) {
		print OUTFILE
		    "#define $ErrorSeverityMacroPrefix\U$symbol",
		    "\t$i\n";
		$i++;
	    }
	}

	if (/^#GENERATE ER_SEV_NAMES$/) {
	    my $symbol;
	    for $symbol (@ErrorLevels) {
		print OUTFILE "\t\"\U$symbol\",\n";
	    }
	}

	if (/^#GENERATE ERRORSEVERITY$/) {
	    my $symbol;
	    for $symbol (@ErrorLevels) {
		print OUTFILE
		    "\t$ErrorSeverityEnumPrefix$symbol\t= ",
		    "$ErrorSeverityMacroPrefix\U$symbol,\n";
	    }
	}

	if (/^#GENERATE SUBSYSTEMS$/) {
	    my $symbol;
	    for $symbol (@Subsystems) {
		$symbol =~ s/=.*//;
		print OUTFILE "\t$ErrorClassPrefix$symbol,\n";
	    }
	    print OUTFILE "\t${ErrorClassPrefix}MaxClass,\n";
	}

	if (/^#GENERATE SUBSYSTEMDEFAULTS$/) {
	    my $symbol;
	    my $value;
	    for $symbol (@Subsystems) {
		# What's behind the "=", if any, otherwise use default
		if ($symbol =~ /=/) {
		    $value = $symbol;
		    $value =~ s/[^=]*=//;
		} else {
		    $value = $DefaultLoggingLevel;
		}
		# The name without the value
		$symbol =~ s/=.*//;
		print OUTFILE
		    "\t{\"$symbol\",",
		    "\t$ErrorSeverityEnumPrefix$value},\n";
	    }
	    print OUTFILE
		"\t{\"$SetAllName\",",
		"\t$ErrorSeverityEnumPrefix$DefaultLoggingLevel}\n";
	}

	if (/^#GENERATE MACROS$/) {
	    # The first macro needs an "And" in the name, because
	    # it logs _and_ aborts
	    my $and="And";
	    my $symbol;
	    for $symbol (@ErrorLevels) {
		print OUTFILE
		    "#define $MacroPrefix\U$symbol\E",
		    "\tif (DEBUG_LEVEL <=",
		    " $ErrorSeverityMacroPrefix\U$symbol\E)",
		    " ErrorLog$and$symbol\n";
		print OUTFILE
		    "#define $MacroPrefix\U$symbol\E_XA",
		    "\tif (DEBUG_LEVEL <=",
		    " $ErrorSeverityMacroPrefix\U$symbol\E)",
		    " ErrorLog$and${symbol}Xa\n";
		# All further entries are without the "And"
		$and = "";
	    }
	}

	if (/^#GENERATE PROTOTYPES$/) {
	    # The first macro needs an "And" in the name, because
	    # it logs _and_ aborts
	    my $and="And";
	    my $noret=", noreturn";
	    my $symbol;
	    for $symbol (@ErrorLevels) {
		print OUTFILE
		    "void\n",
		    "ErrorLog$and$symbol(ErrorClass errorClass,",
		    " ErrorString name,\n",
		    "\tErrorString format, ...)\n",
		    "\t__attribute__((format(printf, 3, 4)$noret));\n",
		    "void\n",
		    "ErrorLog$and${symbol}Xa(ErrorClass errorClass,",
		    " ErrorString name,\n",
		    "\tErrorExtendedLogFunc moreInfo, void *moreData,",
		    "\tErrorString format, ...)\n",
		    "\t__attribute__((format(printf, 5, 6)$noret));\n",
		    # All further entries are without the "And"
		    $and = "";
		$noret = "";
	    }
	}

	if (/^#GENERATE FUNCTIONS$/) {
	    # The first macro needs an "And" in the name, because
	    # it logs _and_ aborts
	    my $and = "And";
	    my $noret = ", noreturn";
	    my $abort = "    ErrorAbort(1,0);\n";
	    my $symbol;
	    for $symbol (@ErrorLevels) {
		print OUTFILE
		    "void\n",
		    "ErrorLog$and$symbol(ErrorClass errorClass,",
		    " ErrorString name,\n",
		    "\tErrorString format, ...)\n",
		    "{\n",
		    "    va_list ap;\n",
		    "    va_start(ap, format);\n",
		    "    ErrorVLogMessage(errorClass, name,",
		    " $ErrorSeverityEnumPrefix$symbol,",
		    " NULL, NULL, format, ap);\n",
		    "    va_end(ap);\n",
		    $abort,
		    "}\n",
		    "void\n",
		    "ErrorLog$and${symbol}Xa(ErrorClass errorClass,",
		    " ErrorString name,\n",
		    "\tErrorExtendedLogFunc moreInfo, void *moreData,",
		    "\tErrorString format, ...)\n",
		    "{\n",
		    "    va_list ap;\n",
		    "    va_start(ap, format);\n",
		    "    ErrorVLogMessage(errorClass, name,",
		    " $ErrorSeverityEnumPrefix$symbol,",
		    " moreInfo, moreData, format, ap);\n",
		    "    va_end(ap);\n",
		    $abort,
		    "}\n";
		# All further entries are without the "And"
		$and = "";
		$noret = "";
		$abort = "";
	    }
	}

	# These are for LogCompatibility.h
	if (/^#GENERATE LOGFACILITIES$/) {
	    my $symbol;
	    for $symbol (@Subsystems) {
		if ($symbol eq $MinLogCompatibility
		    .. $symbol eq $MaxLogCompatibility) {
		    $symbol =~ s/=.*//;
		    print OUTFILE
			"#define LOG_\U$symbol\E",
			"\t($ErrorClassPrefix$symbol << LOG_FACILITY_SHIFT)\n";
		}
	    }
	}

	if (/^#GENERATE LOGPRIORITIES$/) {
	    my $symbol;
	    my $mapping;
	    for $symbol (@ErrorLevels) {
		$mapping=$LogPriorityMapping{$symbol};
		if (defined($mapping)) {
		    print OUTFILE
			"#define $mapping",
			"\t$ErrorSeverityMacroPrefix\U$symbol\E\n";
		}
	    }
	}

	if (/^#GENERATE GENERIC$/) {
	    # This is the generic facility used (says the syslog man page)
	    print OUTFILE "${ErrorClassPrefix}User\n";
	}

	if (/^#GENERATE ABORT$/) {
	    print OUTFILE "${ErrorSeverityEnumPrefix}Abort\n";
	}

	if (/^#GENERATE LOGMIN$/) {
	    print OUTFILE "LOG_\U$MinLogCompatibility\E\n";
	}

	if (/^#GENERATE LOGMAX$/) {
	    print OUTFILE "LOG_\U$MaxLogCompatibility\E\n";
	}

	# Finish line parsing
	if ($_ =~ /^#GENERATE/) {
	    print OUTFILE
		"/* END $_ */\n",
		"#line $linenum \"$infile\"\n";
	} else {
	    print OUTFILE $_, "\n";
	}
    }
    close(INFILE);
    close(OUTFILE);
}
